@@ -412,7 +412,7 @@ tp_base_media_call_content_class_init (TpBaseMediaCallContentClass *klass)
   param_spec = g_param_spec_uchar ("current-dtmf-event",
       "CurrentDTMFEvent",
       "The currently being played dtmf event if any",
-      0, NUM_TP_DTMF_EVENTS - 1, 0,
+      0, TP_NUM_DTMF_EVENTS - 1, 0,
       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CURRENT_DTMF_EVENT,
       param_spec);
@@ -427,7 +427,7 @@ tp_base_media_call_content_class_init (TpBaseMediaCallContentClass *klass)
   param_spec = g_param_spec_uint ("current-dtmf-state",
       "CurrentDTMFState",
       "The sending state of the dtmf events",
-      0, NUM_TP_SENDING_STATES - 1, TP_SENDING_STATE_NONE,
+      0, TP_NUM_SENDING_STATES - 1, TP_SENDING_STATE_NONE,
       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CURRENT_DTMF_STATE,
       param_spec);
@@ -438,7 +438,7 @@ tp_base_media_call_content_class_init (TpBaseMediaCallContentClass *klass)
       "deferred-tones");
 
   /**
-   * TpBaseMediaCallContent::local-media-description-updated
+   * TpBaseMediaCallContent::local-media-description-updated:
    * @self: the #TpCallChannel
    * @contact: the remote contact
    * @properties: the new media description properties asv
@@ -681,22 +681,15 @@ tp_base_media_call_content_update_local_media_description (
   TpBaseMediaCallContent *self = TP_BASE_MEDIA_CALL_CONTENT (iface);
   GHashTable *current_properties;
   GPtrArray *codecs;
-  gpointer contact;
+  TpHandle contact;
+  gboolean valid;
 
-  if (self->priv->current_offer != NULL)
-    {
-      GError error = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-          "There is a media description offer around so "
-          "UpdateMediaDescription shouldn't be called." };
-      dbus_g_method_return_error (context, &error);
-      return;
-    }
+  contact = tp_asv_get_uint32 (properties,
+      TP_PROP_CALL_CONTENT_MEDIA_DESCRIPTION_REMOTE_CONTACT, &valid);
 
-  if (!g_hash_table_lookup_extended (properties,
-          TP_PROP_CALL_CONTENT_MEDIA_DESCRIPTION_REMOTE_CONTACT,
-          NULL, &contact))
+  if (!valid)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError error = { TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "The media description is missing the RemoteContact key." };
       dbus_g_method_return_error (context, &error);
       return;
@@ -704,11 +697,11 @@ tp_base_media_call_content_update_local_media_description (
 
   current_properties = g_hash_table_lookup (
       self->priv->local_media_descriptions,
-      contact);
+      GUINT_TO_POINTER (contact));
 
   if (current_properties == NULL)
     {
-      GError error = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+      GError error = { TP_ERROR, TP_ERROR_NOT_AVAILABLE,
           "The initial MediaDescription object has not yet appeared" };
       dbus_g_method_return_error (context, &error);
       return;
@@ -720,23 +713,23 @@ tp_base_media_call_content_update_local_media_description (
       TP_ARRAY_TYPE_CODEC_LIST);
   if (!codecs || codecs->len == 0)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError error = { TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
                        "Codecs can not be empty" };
       dbus_g_method_return_error (context, &error);
       return;
     }
 
   if (self->priv->current_offer != NULL &&
-      tp_call_content_media_description_get_remote_contact (self->priv->current_offer) == GPOINTER_TO_UINT (contact))
+      tp_call_content_media_description_get_remote_contact (self->priv->current_offer) == contact)
     {
-      GError error = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+      GError error = { TP_ERROR, TP_ERROR_NOT_AVAILABLE,
                        "Can not update the media description while there is"
                        " an outstanding offer for this contact." };
       dbus_g_method_return_error (context, &error);
       return;
     }
 
-  set_local_properties (self, GPOINTER_TO_UINT (contact), properties);
+  set_local_properties (self, contact, properties);
 
   tp_svc_call_content_interface_media_return_from_update_local_media_description
       (context);
@@ -769,7 +762,7 @@ tp_base_media_call_content_acknowledge_dtmf_change (
 
   if (self->priv->current_dtmf_event != in_Event)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError error = { TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "The acknoledgement is not for the right event"};
       dbus_g_method_return_error (context, &error);
       return;
@@ -778,7 +771,7 @@ tp_base_media_call_content_acknowledge_dtmf_change (
   if (in_State != TP_SENDING_STATE_SENDING &&
       in_State != TP_SENDING_STATE_NONE)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError error = { TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "The new sending state can not be a pending state"};
       dbus_g_method_return_error (context, &error);
       return;
@@ -790,7 +783,7 @@ tp_base_media_call_content_acknowledge_dtmf_change (
   if (self->priv->current_dtmf_state != TP_SENDING_STATE_PENDING_SEND &&
       self->priv->current_dtmf_state != TP_SENDING_STATE_PENDING_STOP_SENDING)
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError error = { TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "Acknowledge rejected because we are not in a pending state"};
       dbus_g_method_return_error (context, &error);
       return;
@@ -802,7 +795,7 @@ tp_base_media_call_content_acknowledge_dtmf_change (
           TP_SENDING_STATE_PENDING_STOP_SENDING &&
           in_State != TP_SENDING_STATE_NONE))
     {
-      GError error = { TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      GError error = { TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "The new sending state does not match the pending state"};
       dbus_g_method_return_error (context, &error);
       return;
@@ -934,7 +927,7 @@ tp_base_media_call_content_start_tone (TpBaseCallContent *bcc,
 
   if (self->priv->currently_sending_tones != NULL)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_SERVICE_BUSY,
+      g_set_error (error, TP_ERROR, TP_ERROR_SERVICE_BUSY,
           "Already sending a tone");
       return FALSE;
     }
@@ -976,7 +969,7 @@ tp_base_media_call_content_stop_tone (TpBaseCallContent *bcc,
 
   if (self->priv->currently_sending_tones == NULL)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      g_set_error (error, TP_ERROR, TP_ERROR_INVALID_ARGUMENT,
           "No tone is currently being played");
       return FALSE;
     }
@@ -1007,7 +1000,7 @@ tp_base_media_call_content_multiple_tones (TpBaseCallContent *bcc,
 
   if (self->priv->currently_sending_tones != NULL)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_SERVICE_BUSY,
+      g_set_error (error, TP_ERROR, TP_ERROR_SERVICE_BUSY,
           "Already sending a tone");
       return FALSE;
     }
