@@ -143,6 +143,7 @@ struct _TpProtocolPrivate
   GStrv authentication_types;
   TpCapabilities *capabilities;
   TpAvatarRequirements *avatar_req;
+  gchar *cm_name;
 };
 
 enum
@@ -156,6 +157,7 @@ enum
     PROP_PARAM_NAMES,
     PROP_AUTHENTICATION_TYPES,
     PROP_AVATAR_REQUIREMENTS,
+    PROP_CM_NAME,
     N_PROPS
 };
 
@@ -286,6 +288,10 @@ tp_protocol_get_property (GObject *object,
       g_value_set_pointer (value, tp_protocol_get_avatar_requirements (self));
       break;
 
+    case PROP_CM_NAME:
+      g_value_set_string (value, tp_protocol_get_cm_name (self));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -312,6 +318,11 @@ tp_protocol_set_property (GObject *object,
       self->priv->protocol_properties = g_value_dup_boxed (value);
       break;
 
+    case PROP_CM_NAME:
+      g_assert (self->priv->cm_name == NULL);
+      self->priv->cm_name = g_value_dup_string (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -381,6 +392,7 @@ tp_protocol_finalize (GObject *object)
   g_free (self->priv->vcard_field);
   g_free (self->priv->english_name);
   g_free (self->priv->icon_name);
+  g_free (self->priv->cm_name);
 
   if (self->priv->protocol_properties != NULL)
     g_hash_table_unref (self->priv->protocol_properties);
@@ -737,6 +749,20 @@ tp_protocol_class_init (TpProtocolClass *klass)
         "Avatars requirements",
         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * TpProtocol:cm-name:
+   *
+   * The name of the connection manager this protocol is on.
+   *
+   * Since: 0.19.1
+   */
+  g_object_class_install_property (object_class, PROP_CM_NAME,
+      g_param_spec_string ("cm-name",
+        "Connection manager name",
+        "Name of the CM this protocol is on",
+        NULL,
+        G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   proxy_class->list_features = tp_protocol_list_features;
   proxy_class->must_have_unique_name = FALSE;
   proxy_class->interface = TP_IFACE_QUARK_PROTOCOL;
@@ -795,6 +821,7 @@ tp_protocol_new (TpDBusDaemon *dbus,
         "object-path", object_path,
         "protocol-name", protocol_name,
         "protocol-properties", immutable_properties,
+        "cm-name", cm_name,
         NULL));
 
 finally:
@@ -829,7 +856,7 @@ tp_protocol_init_known_interfaces (void)
       tp_proxy_or_subclass_hook_on_interface_add (type,
           tp_cli_protocol_add_signals);
       tp_proxy_subclass_add_error_mapping (type,
-          TP_ERROR_PREFIX, TP_ERRORS, TP_TYPE_ERROR);
+          TP_ERROR_PREFIX, TP_ERROR, TP_TYPE_ERROR);
 
       g_once_init_leave (&once, 1);
     }
@@ -877,7 +904,6 @@ tp_protocol_has_param (TpProtocol *self,
   return (tp_protocol_get_param (self, param) != NULL);
 }
 
-/* FIXME: in Telepathy 1.0, rename to tp_protocol_borrow_param or remove */
 /**
  * tp_protocol_get_param:
  * @self: a protocol
@@ -890,15 +916,17 @@ tp_protocol_has_param (TpProtocol *self,
  *
  * Since: 0.11.11
  */
-const TpConnectionManagerParam *tp_protocol_get_param (TpProtocol *self,
+const TpConnectionManagerParam *
+tp_protocol_get_param (TpProtocol *self,
     const gchar *param)
 {
   g_return_val_if_fail (TP_IS_PROTOCOL (self), FALSE);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   return tp_connection_manager_protocol_get_param (
       &self->priv->protocol_struct, param);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
-/* FIXME: in Telepathy 1.0, rename to tp_protocol_get_param */
 /**
  * tp_protocol_dup_param:
  * @self: a protocol
@@ -917,9 +945,7 @@ tp_protocol_dup_param (TpProtocol *self,
 {
   g_return_val_if_fail (TP_IS_PROTOCOL (self), NULL);
 
-  return tp_connection_manager_param_copy (
-      tp_connection_manager_protocol_get_param (
-        &self->priv->protocol_struct, param));
+  return tp_connection_manager_param_copy (tp_protocol_get_param (self, param));
 }
 
 /**
@@ -948,7 +974,7 @@ tp_protocol_can_register (TpProtocol *self)
  *
  * The result is copied and must be freed by the caller with g_strfreev().
  *
- * Returns: (type GObject.Strv) (transfer full): a copy of
+ * Returns: (array zero-terminated=1) (transfer full): a copy of
  *  #TpProtocol:param-names
  *
  * Since: 0.11.11
@@ -957,8 +983,10 @@ GStrv
 tp_protocol_dup_param_names (TpProtocol *self)
 {
   g_return_val_if_fail (TP_IS_PROTOCOL (self), NULL);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   return tp_connection_manager_protocol_dup_param_names (
       &self->priv->protocol_struct);
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 /**
@@ -973,6 +1001,8 @@ tp_protocol_dup_param_names (TpProtocol *self)
  *  terminated by one whose @name is %NULL
  *
  * Since: 0.17.6
+ * Deprecated: Since 0.19.9. New code should use tp_protocol_dup_params()
+ *  instead.
  */
 const TpConnectionManagerParam *
 tp_protocol_borrow_params (TpProtocol *self)
@@ -1696,3 +1726,21 @@ tp_protocol_get_avatar_requirements (TpProtocol *self)
 
   return self->priv->avatar_req;
 }
+
+/**
+ * tp_protocol_get_cm_name:
+ * @self: a #TpProtocol
+ *
+ * Return the #TpProtocol:cm-name property.
+ *
+ * Returns: the value of #TpProtocol:cm-name
+ *
+ * Since: 0.19.1
+ */
+const gchar *
+tp_protocol_get_cm_name (TpProtocol *self)
+{
+  g_return_val_if_fail (TP_IS_PROTOCOL (self), NULL);
+
+  return self->priv->cm_name;
+}
